@@ -25,6 +25,7 @@ use Composer\Semver\Constraint\Constraint;
 use Composer\Semver\Constraint\ConstraintInterface;
 use Composer\Package\Version\VersionParser;
 use Composer\Repository\PlatformRepository;
+use Composer\Semver\Constraint\MultiConstraint;
 
 /**
  * Represents a problem detected while solving dependencies
@@ -79,7 +80,7 @@ class Problem
         // TODO doesn't this entirely defeat the purpose of the problem sections? what's the point of sections?
         $reasons = array_merge(...array_reverse($this->reasons));
 
-        if (count($reasons) === 1) {
+        if (\count($reasons) === 1) {
             reset($reasons);
             $rule = current($reasons);
 
@@ -92,14 +93,65 @@ class Problem
             $constraint = $reasonData['constraint'];
 
             $packages = $pool->whatProvides($packageName, $constraint);
-            if (count($packages) === 0) {
+            if (\count($packages) === 0) {
                 return "\n    ".implode(self::getMissingPackageReason($repositorySet, $request, $pool, $isVerbose, $packageName, $constraint));
             }
         }
 
+        usort($reasons, function (Rule $rule1, Rule $rule2) use ($pool) {
+            $rule1Prio = $this->getRulePriority($rule1);
+            $rule2Prio = $this->getRulePriority($rule2);
+            if ($rule1Prio !== $rule2Prio) {
+                return $rule2Prio - $rule1Prio;
+            }
+
+            return $this->getSortableString($pool, $rule1) <=> $this->getSortableString($pool, $rule2);
+        });
+
         return self::formatDeduplicatedRules($reasons, '    ', $repositorySet, $request, $pool, $isVerbose, $installedMap, $learnedPool);
     }
 
+    private function getSortableString(Pool $pool, Rule $rule): string
+    {
+        switch ($rule->getReason()) {
+            case Rule::RULE_ROOT_REQUIRE:
+                return $rule->getReasonData()['packageName'];
+            case Rule::RULE_FIXED:
+                return (string) $rule->getReasonData()['package'];
+            case Rule::RULE_PACKAGE_CONFLICT:
+            case Rule::RULE_PACKAGE_REQUIRES:
+                return $rule->getSourcePackage($pool) . '//' . $rule->getReasonData()->getPrettyString($rule->getSourcePackage($pool));
+            case Rule::RULE_PACKAGE_SAME_NAME:
+            case Rule::RULE_PACKAGE_ALIAS:
+            case Rule::RULE_PACKAGE_INVERSE_ALIAS:
+                return (string) $rule->getReasonData();
+            case Rule::RULE_LEARNED:
+                return implode('-', $rule->getLiterals());
+        }
+
+        throw new \LogicException('Unknown rule type: '.$rule->getReason());
+    }
+
+    private function getRulePriority(Rule $rule): int
+    {
+        switch ($rule->getReason()) {
+            case Rule::RULE_FIXED:
+                return 3;
+            case Rule::RULE_ROOT_REQUIRE:
+                return 2;
+            case Rule::RULE_PACKAGE_CONFLICT:
+            case Rule::RULE_PACKAGE_REQUIRES:
+                return 1;
+            case Rule::RULE_PACKAGE_SAME_NAME:
+            case Rule::RULE_LEARNED:
+            case Rule::RULE_PACKAGE_ALIAS:
+            case Rule::RULE_PACKAGE_INVERSE_ALIAS:
+                return 0;
+        }
+
+        throw new \LogicException('Unknown rule type: '.$rule->getReason());
+    }
+
     /**
      * @param Rule[] $rules
      * @param array<int|string, BasePackage> $installedMap A map of all present packages
@@ -136,7 +188,7 @@ class Problem
                     if (!$isVerbose) {
                         $versions = self::condenseVersionList($versions, 1);
                     }
-                    if (count($versions) > 1) {
+                    if (\count($versions) > 1) {
                         // remove the s from requires/conflicts to correct grammar
                         $message = Preg::replace('{^(%s%s (?:require|conflict))s}', '$1', $message);
                         $result[] = sprintf($message, $package, '['.implode(', ', $versions).']');
@@ -224,16 +276,21 @@ class Problem
                 $ext = substr($packageName, 4);
                 $msg = "- Root composer.json requires PHP extension ".$packageName.self::constraintToText($constraint).' but ';
 
-                $version = self::getPlatformPackageVersion($pool, $packageName, phpversion($ext) ?: '0');
+                $version = self::getPlatformPackageVersion($pool, $packageName, phpversion($ext) === false ? '0' : phpversion($ext));
                 if (null === $version) {
+                    $providersStr = self::getProvidersList($repositorySet, $packageName, 5);
+                    if ($providersStr !== null) {
+                        $providersStr = "\n\n      Alternatively you can require one of these packages that provide the extension (or parts of it):\n$providersStr";
+                    }
+
                     if (extension_loaded($ext)) {
                         return [
                             $msg,
-                            'the '.$packageName.' package is disabled by your platform config. Enable it again with "composer config platform.'.$packageName.' --unset".',
+                            'the '.$packageName.' package is disabled by your platform config. Enable it again with "composer config platform.'.$packageName.' --unset".' . $providersStr,
                         ];
                     }
 
-                    return [$msg, 'it is missing from your system. Install or enable PHP\'s '.$ext.' extension.'];
+                    return [$msg, 'it is missing from your system. Install or enable PHP\'s '.$ext.' extension.' . $providersStr];
                 }
 
                 return [$msg, 'it has the wrong version installed ('.$version.').'];
@@ -247,7 +304,12 @@ class Problem
                     return ["- Root composer.json requires linked library ".$packageName.self::constraintToText($constraint).' but ', $error];
                 }
 
-                return ["- Root composer.json requires linked library ".$packageName.self::constraintToText($constraint).' but ', 'it has the wrong version installed or is missing from your system, make sure to load the extension providing it.'];
+                $providersStr = self::getProvidersList($repositorySet, $packageName, 5);
+                if ($providersStr !== null) {
+                    $providersStr = "\n\n      Alternatively you can require one of these packages that provide the library (or parts of it):\n$providersStr";
+                }
+
+                return ["- Root composer.json requires linked library ".$packageName.self::constraintToText($constraint).' but ', 'it has the wrong version installed or is missing from your system, make sure to load the extension providing it.'.$providersStr];
             }
         }
 
@@ -262,9 +324,21 @@ class Problem
             }
         }
 
+        if ($constraint instanceof Constraint && $constraint->getOperator() === Constraint::STR_OP_EQ && Preg::isMatch('{^dev-.*#.*}', $constraint->getPrettyString())) {
+            $newConstraint = Preg::replace('{ +as +([^,\s|]+)$}', '', $constraint->getPrettyString());
+            $packages = $repositorySet->findPackages($packageName, new MultiConstraint([
+                new Constraint(Constraint::STR_OP_EQ, $newConstraint),
+                new Constraint(Constraint::STR_OP_EQ, str_replace('#', '+', $newConstraint))
+            ], false));
+            if (\count($packages) > 0) {
+                return ["- Root composer.json requires $packageName".self::constraintToText($constraint) . ', ', 'found '.self::getPackageList($packages, $isVerbose, $pool, $constraint).'. The # character in branch names is replaced by a + character. Make sure to require it as "'.str_replace('#', '+', $constraint->getPrettyString()).'".'];
+            }
+        }
+
         // first check if the actual requested package is found in normal conditions
         // if so it must mean it is rejected by another constraint than the one given here
-        if ($packages = $repositorySet->findPackages($packageName, $constraint)) {
+        $packages = $repositorySet->findPackages($packageName, $constraint);
+        if (\count($packages) > 0) {
             $rootReqs = $repositorySet->getRootRequires();
             if (isset($rootReqs[$packageName])) {
                 $filtered = array_filter($packages, static function ($p) use ($rootReqs, $packageName): bool {
@@ -285,7 +359,7 @@ class Problem
                 }
             }
 
-            if ($lockedPackage) {
+            if ($lockedPackage !== null) {
                 $fixedConstraint = new Constraint('==', $lockedPackage->getVersion());
                 $filtered = array_filter($packages, static function ($p) use ($fixedConstraint): bool {
                     return $fixedConstraint->matches(new Constraint('==', $p->getVersion()));
@@ -299,7 +373,7 @@ class Problem
                 return !$p->getRepository() instanceof LockArrayRepository;
             });
 
-            if (!$nonLockedPackages) {
+            if (0 === \count($nonLockedPackages)) {
                 return ["- Root composer.json requires $packageName".self::constraintToText($constraint) . ', ', 'found '.self::getPackageList($packages, $isVerbose, $pool, $constraint).' in the lock file but not in remote repositories, make sure you avoid updating this package to keep the one from the lock file.'];
             }
 
@@ -307,9 +381,11 @@ class Problem
         }
 
         // check if the package is found when bypassing stability checks
-        if ($packages = $repositorySet->findPackages($packageName, $constraint, RepositorySet::ALLOW_UNACCEPTABLE_STABILITIES)) {
+        $packages = $repositorySet->findPackages($packageName, $constraint, RepositorySet::ALLOW_UNACCEPTABLE_STABILITIES);
+        if (\count($packages) > 0) {
             // we must first verify if a valid package would be found in a lower priority repository
-            if ($allReposPackages = $repositorySet->findPackages($packageName, $constraint, RepositorySet::ALLOW_SHADOWED_REPOSITORIES)) {
+            $allReposPackages = $repositorySet->findPackages($packageName, $constraint, RepositorySet::ALLOW_SHADOWED_REPOSITORIES);
+            if (\count($allReposPackages) > 0) {
                 return self::computeCheckForLowerPrioRepo($pool, $isVerbose, $packageName, $packages, $allReposPackages, 'minimum-stability', $constraint);
             }
 
@@ -317,9 +393,11 @@ class Problem
         }
 
         // check if the package is found when bypassing the constraint and stability checks
-        if ($packages = $repositorySet->findPackages($packageName, null, RepositorySet::ALLOW_UNACCEPTABLE_STABILITIES)) {
+        $packages = $repositorySet->findPackages($packageName, null, RepositorySet::ALLOW_UNACCEPTABLE_STABILITIES);
+        if (\count($packages) > 0) {
             // we must first verify if a valid package would be found in a lower priority repository
-            if ($allReposPackages = $repositorySet->findPackages($packageName, $constraint, RepositorySet::ALLOW_SHADOWED_REPOSITORIES)) {
+            $allReposPackages = $repositorySet->findPackages($packageName, $constraint, RepositorySet::ALLOW_SHADOWED_REPOSITORIES);
+            if (\count($allReposPackages) > 0) {
                 return self::computeCheckForLowerPrioRepo($pool, $isVerbose, $packageName, $packages, $allReposPackages, 'constraint', $constraint);
             }
 
@@ -349,17 +427,8 @@ class Problem
             return ["- Root composer.json requires $packageName, it ", 'could not be found, it looks like its name is invalid, "'.$illegalChars.'" is not allowed in package names.'];
         }
 
-        if ($providers = $repositorySet->getProviders($packageName)) {
-            $maxProviders = 20;
-            $providersStr = implode(array_map(static function ($p): string {
-                $description = $p['description'] ? ' '.substr($p['description'], 0, 100) : '';
-
-                return '      - '.$p['name'].$description."\n";
-            }, count($providers) > $maxProviders + 1 ? array_slice($providers, 0, $maxProviders) : $providers));
-            if (count($providers) > $maxProviders + 1) {
-                $providersStr .= '      ... and '.(count($providers) - $maxProviders).' more.'."\n";
-            }
-
+        $providersStr = self::getProvidersList($repositorySet, $packageName, 15);
+        if ($providersStr !== null) {
             return ["- Root composer.json requires $packageName".self::constraintToText($constraint).", it ", "could not be found in any version, but the following packages provide it:\n".$providersStr."      Consider requiring one of these to satisfy the $packageName requirement."];
         }
 
@@ -377,12 +446,12 @@ class Problem
         foreach ($packages as $package) {
             $prepared[$package->getName()]['name'] = $package->getPrettyName();
             $prepared[$package->getName()]['versions'][$package->getVersion()] = $package->getPrettyVersion().($package instanceof AliasPackage ? ' (alias of '.$package->getAliasOf()->getPrettyVersion().')' : '');
-            if ($pool && $constraint) {
+            if ($pool !== null && $constraint !== null) {
                 foreach ($pool->getRemovedVersions($package->getName(), $constraint) as $version => $prettyVersion) {
                     $prepared[$package->getName()]['versions'][$version] = $prettyVersion;
                 }
             }
-            if ($pool && $useRemovedVersionGroup) {
+            if ($pool !== null && $useRemovedVersionGroup) {
                 foreach ($pool->getRemovedVersionsByPackage(spl_object_hash($package)) as $version => $prettyVersion) {
                     $prepared[$package->getName()]['versions'][$version] = $prettyVersion;
                 }
@@ -418,7 +487,7 @@ class Problem
     {
         $available = $pool->whatProvides($packageName);
 
-        if (count($available)) {
+        if (\count($available) > 0) {
             $selected = null;
             foreach ($available as $pkg) {
                 if ($pkg->getRepository() instanceof PlatformRepository) {
@@ -443,7 +512,7 @@ class Problem
             $version = $selected->getPrettyVersion();
             $extra = $selected->getExtra();
             if ($selected instanceof CompletePackageInterface && isset($extra['config.platform']) && $extra['config.platform'] === true) {
-                $version .= '; ' . str_replace('Package ', '', $selected->getDescription());
+                $version .= '; ' . str_replace('Package ', '', (string) $selected->getDescription());
             }
         } else {
             return null;
@@ -504,8 +573,8 @@ class Problem
     }
 
     /**
-     * @param PackageInterface[] $higherRepoPackages
-     * @param PackageInterface[] $allReposPackages
+     * @param non-empty-array<PackageInterface> $higherRepoPackages
+     * @param non-empty-array<PackageInterface> $allReposPackages
      * @return array{0: string, 1: string}
      */
     private static function computeCheckForLowerPrioRepo(Pool $pool, bool $isVerbose, string $packageName, array $higherRepoPackages, array $allReposPackages, string $reason, ?ConstraintInterface $constraint = null): array
@@ -522,7 +591,9 @@ class Problem
             }
         }
 
-        if ($higherRepoPackages) {
+        assert(null !== $nextRepo);
+
+        if (\count($higherRepoPackages) > 0) {
             $topPackage = reset($higherRepoPackages);
             if ($topPackage instanceof RootPackageInterface) {
                 return [
@@ -570,6 +641,25 @@ class Problem
             return ' ' . $constraint->getPrettyString() . ' (exact version match: ' . (count($versions) > 1 ? implode(', ', array_slice($versions, 0, -1)) . ' or ' . end($versions) : $versions[0]) . ')';
         }
 
-        return $constraint ? ' '.$constraint->getPrettyString() : '';
+        return $constraint !== null ? ' '.$constraint->getPrettyString() : '';
+    }
+
+    private static function getProvidersList(RepositorySet $repositorySet, string $packageName, int $maxProviders): ?string
+    {
+        $providers = $repositorySet->getProviders($packageName);
+        if (\count($providers) > 0) {
+            $providersStr = implode(array_map(static function ($p): string {
+                $description = $p['description'] !== '' ? ' '.substr($p['description'], 0, 100) : '';
+
+                return '      - '.$p['name'].$description."\n";
+            }, count($providers) > $maxProviders + 1 ? array_slice($providers, 0, $maxProviders) : $providers));
+            if (count($providers) > $maxProviders + 1) {
+                $providersStr .= '      ... and '.(count($providers) - $maxProviders).' more.'."\n";
+            }
+
+            return $providersStr;
+        }
+
+        return null;
     }
 }
